function data_out = compressQuicklz(data_in)
% compressQuicklz - compress Matlab array to byte vector using QuickLz
%
% CALL:
%  data_out = compressQuicklz(data_in,level)
%
% INPUT
%     data_in(M,N): input array with numeric, charater or logical data
%                   (cell arrays, structure arrays etc not allowed)
%
% OUTPUT:
%  data_out(P,1): int8 vector that contains compressed variable
%
%
% REMARKS:
%  When an array is compressed the size and class of the
%  original array can not be inferred from data_out. In order to
%  succesfully inflate the array again. uncompress requires the size
%  and class of the original array.
%
% EXAMPLE:
%  %create an example array with N elements
%  a=int16(100*rand(N,2000,100)); %or any ather type
%  aa = compress(a);
%  aaa =uncompress(aa,size(a),class(a));
%
% SEE ALSO:
%     uncompressQuicklz

% Copyright (C) 2010
% Nanne van der Zijpp
% Modelit
% www.modelit.nl

if nargin==0
    bench
    return
end
if nargin==0
    error('At least 1 input argument required')
end
if islogical(data_in)
    data_in=precompress(data_in);
end
data_out=compress_mex_quicklz(data_in);
%__________________________________________________________________________
function data=precompress(data)
% pack logical in uint8.
% number of elements may not be 8-multiple
numbytes=numel(data)/8;
if rem(numbytes,1)
    %pad with false values
    numbytes=ceil(numbytes);
    data=data(:); %prevent error on matrix
    data((end+1):8*numbytes)=false;
end
data=logical2int(reshape(data,8,numbytes));
%__________________________________________________________________________
function bench
rand('seed',1);
data=sort(rand(800000,1));
types={'logical','uint8','uint16','uint32','single','double'};
%  types={'uint8','uint16','uint32','single','double'};
disp(' ')
for k=1:length(types)
    type=types{k};
    switch type
        case 'logical'
            D=data>.5;
        case {'uint8','uint16','uint32'}
            D=feval(type,double(intmax(type))*data);
        otherwise
            D=feval(type,data);
    end
    tic;
    dComp1=compressQuicklz(D);
    t1=toc;

    tic
    D1=uncompressQuicklz(dComp1,size(D),type);
    t2=toc;
    if ~all(size(D1)==size(D))||~all(D1(:)==D(:))
        error('Data changed in compression-decromression cycle');
    end

    fprintf('Class=%7s compress: %.4f sec ==> CmpRate %6.2f%% uncompress: %.4f sec\n',...
        type,t1,100*numel(dComp1)/(numel(D)*sizeof(type)),t2);
end


